#!/usr/bin/env python
"""
_WMComponent_

Contains generic components that can be
used cross project.

Project specific component should be developed
within its own project space (not in wmcomponent).

"""
__all__ = []
