#!/usr/bin/env python
"""
_SucceededJobs_

Oracle implementation of Subscriptions.SucceededJobs
"""




from WMCore.WMBS.MySQL.Subscriptions.SucceededJobs import SucceededJobs as SucceededJobsMySQL

class SucceededJobs(SucceededJobsMySQL):
    pass
