#!/usr/bin/env python
"""
_CountFilesByStatus_

Oracle implementation of DBSBuffer.CountFilesByStatus
"""

from __future__ import division
from WMComponent.DBS3Buffer.MySQL.CountFilesByStatus import CountFilesByStatus as MySQLCountFilesByStatus

class CountFilesByStatus(MySQLCountFilesByStatus):
    """
    CountFilesByStatus

    """
    pass
