#!/usr/bin/env python
"""
_GetType_

Oracle implementation of Jobs.GetType
"""




from WMCore.WMBS.MySQL.Jobs.GetType import GetType as MySQLGetType

class GetType(MySQLGetType):
    """
    Identical to MySQL version for now

    """
    pass
